package com.utopiaindustries.controller;

import com.utopiaindustries.model.ctp.JobCard;
import com.utopiaindustries.service.JobCardService;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

@Controller
@RequestMapping( "/job-cards" )
public class JobCardController {

    private final JobCardService jobCardService;

    public JobCardController(JobCardService jobCardService){
        this.jobCardService = jobCardService;
    }

    /**
     *  get all job cards
     * */
    @GetMapping
    public String showJobCardList( Model model ){
        model.addAttribute("cards", jobCardService.getCards() );
        return "job-card-list";
    }

    @GetMapping( "/new" )
    public String showJobCardForm( Model model ){
        model.addAttribute("jobCard", jobCardService.createNewJobCard() );
        model.addAttribute("cutPieceTypes", jobCardService.getAllPieceTypes() );
        return "job-card-add";
    }

    @GetMapping( value = "/edit/{id}" )
    public String showJobCardEditForm( @PathVariable("id") long id,
                                       Model model ){
        model.addAttribute("jobCard", jobCardService.findCardRecursively( id ) );
        model.addAttribute("cutPieceTypes", jobCardService.getAllPieceTypes() );
        return "job-card-edit";
    }

    /*
    *  draft
    * */
    @PostMapping( value ="/edit" , params = "user=draft" )
    public String saveJobCard( @ModelAttribute JobCard jobCard,
                               RedirectAttributes redirectAttributes,
                               Model model ){

        try {
            jobCard.setStatus( JobCard.Status.DRAFT.name() );
            jobCard.setInventoryStatus( JobCard.InventoryStatus.NOT_RECEIVED_YET.name() );
            jobCardService.save( jobCard );
           redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }

    /*
    *  draft after save
    * */
    @PostMapping( value ="/edit/{id}" , params = "user=draft" )
    public String saveJobCardEdit( @ModelAttribute JobCard jobCard,
                                   @PathVariable long id,
                                   RedirectAttributes redirectAttributes,
                                   Model model ){

        try {
            jobCard.setStatus( JobCard.Status.DRAFT.name() );
            jobCard.setInventoryStatus( JobCard.InventoryStatus.NOT_RECEIVED_YET.name() );
            jobCardService.save( jobCard );
            redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }

    @PostMapping( value ="/edit" , params = "user=post" )
    public String postJobCard( @ModelAttribute JobCard jobCard,
                               RedirectAttributes redirectAttributes,
                               Model model ){

        try {
            jobCard.setStatus(JobCard.Status.POSTED.name() );
            jobCardService.save( jobCard );
            redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }


    /*
    * post after draft
    * */
    @PostMapping( value ="/edit/{id}" , params = "user=post" )
    public String postJobCardEdit( @ModelAttribute JobCard jobCard,
                                   @PathVariable long id,
                                   RedirectAttributes redirectAttributes,
                                   Model model ){

        try {
            jobCard.setStatus(JobCard.Status.POSTED.name() );
            jobCardService.save( jobCard );
            redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }
}
