package com.utopiaindustries.controller;

import com.utopiaindustries.model.ctp.JobCard;
import com.utopiaindustries.model.ctp.StitchingOfflineItem;
import com.utopiaindustries.service.*;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.util.Arrays;
import java.util.List;

@Controller
@RequestMapping( "/stitching" )
public class StitchingController {

    private final InventoryAccountService inventoryAccountService;
    private final JobCardService jobCardService;
    private final InventoryService inventoryService;
    private final BundleService bundleService;
    private final BarcodeService barcodeService;
    private final LocationService locationService;

    public StitchingController(InventoryAccountService inventoryAccountService, JobCardService jobCardService, InventoryService inventoryService, BundleService bundleService, BarcodeService barcodeService, LocationService locationService) {
        this.inventoryAccountService = inventoryAccountService;
        this.jobCardService = jobCardService;
        this.inventoryService = inventoryService;
        this.bundleService = bundleService;
        this.barcodeService = barcodeService;
        this.locationService = locationService;
    }

    @GetMapping
    public String homePage( Model model ){
        return "redirect:/stitching/receive-inventory";
    }

    @GetMapping( "/receive-inventory" )
    public String receiveInventoryForm( Model model ){
        // 2 for stitching
        model.addAttribute("accounts" , inventoryAccountService.findInventoryAccounts( 2L ) );
        model.addAttribute("cutPieceTypes", jobCardService.getAllPieceTypes() );
        return "/stitching/receive-inventory";
    }

    @PostMapping( "/receive-inventory" )
    public String receiveInventoryToCuttingAccount( RedirectAttributes redirectAttributes,
                                                    @RequestParam( "master-id" ) long masterId,
                                                    @RequestParam( "account-id" ) long accountId ){
        try {
            inventoryService.receiveInventoryFromMasterBarcode( masterId, accountId );
            redirectAttributes.addFlashAttribute("success", "Inventory Success Received" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/stitching/receive-inventory";
    }

    /*
     *  get stitching inventory accounts
     * */
    @GetMapping( "/inventory-accounts" )
    public String getInventoryAccounts(  @RequestParam( value = "id", required = false ) String id,
                                         @RequestParam( value = "title", required = false) String title,
                                         @RequestParam( value = "active", required = false ) String active,
                                         @RequestParam( value = "created-by", required = false ) String createdBy,
                                         @RequestParam( value = "start-date", required = false ) String startDate,
                                         @RequestParam( value = "end-date", required = false ) String endDate,
                                         @RequestParam( value = "site-id", required = false ) String siteId,
                                         @RequestParam( value = "count", required = false ) Long count,
                                         Model model ) {
        // 2 for stitching
        model.addAttribute("accounts", inventoryAccountService.getInventoryAccounts(id, title, active, createdBy, startDate, endDate, siteId, count, "PROCESS", "2", false));
        model.addAttribute("locations", locationService.findAll() );
        return "/stitching/inventory-accounts";
    }

    /*
    * get finished items
    * */
    @GetMapping( "/stitching-offline-items" )
    public String getStitchingOfflineItems( @RequestParam(value = "id", required = false ) String id,
                                            @RequestParam(value = "item-id", required = false ) String itemId,
                                            @RequestParam( value = "sku", required = false ) String sku,
                                            @RequestParam( value = "start-date", required = false) String startDate,
                                            @RequestParam( value = "end-date", required = false ) String endDate,
                                            @RequestParam( value = "job-card-id", required = false ) String jobCardId,
                                            @RequestParam( value = "count", required = false ) Long count,
                                            Model model ){
        List<StitchingOfflineItem> itemList = bundleService.getStitchedOfflineItems( id, itemId, sku, startDate, endDate, jobCardId ,count );
        model.addAttribute("items", itemList ) ;
        return "/stitching/stitched-offline-items";
    }

    @GetMapping( "/create-stitching-items")
    public String createFinishItems( Model model ){
        model.addAttribute("accounts" , inventoryAccountService.findInventoryAccounts( 2L ) );
        model.addAttribute("jobCard", jobCardService.createNewJobCard() );
        return "/stitching/stitching-item-form";
    }

    @PostMapping( "/create-stitching-items" )
    public String createStitchedOfflineItems( @ModelAttribute JobCard jobCard,
                                               RedirectAttributes redirectAttributes,
                                               Model model ){
        try {
            inventoryService.createStitchingOfflineItemsFromJobCard( jobCard );
            redirectAttributes.addFlashAttribute("success", "Finished Item Created Successfully");
        } catch ( Exception exception ){
            redirectAttributes.addFlashAttribute( "error", exception.getMessage() );
        }
        return "redirect:/stitching/stitching-offline-items";
    }

    @PostMapping( "/generate-barcodes" )
    public ResponseEntity<InputStreamResource> generateBarcode(@RequestParam( name = "ids"  ) Long[] ids,
                                                               @RequestParam( name = "artifactType"  ) String artifactType ) throws Exception {

        return barcodeService.generateBarcodes( Arrays.asList( ids ), artifactType );
    }
}
